@@ -8,7 +8,7 @@
 #' @importFrom utils globalVariables packageVersion
 #' @section Main resources:
 #' * Book on mlr3: \url{https://mlr3book.mlr-org.com}
-#' * mlr3book section about spatiotemporal data: \url{https://mlr3book.mlr-org.com/special.html#spatiotemporal}
+#' * mlr3book section about spatiotemporal data: \url{https://mlr3book.mlr-org.com/chapters/chapter13/beyond_regression_and_classification.html#spatiotemp-cv}
 #' * package vignettes: \url{https://mlr3spatiotempcv.mlr-org.com/dev/articles/}
 #'
 #' ## Miscellaneous \pkg{mlr3} content
@@ -106,6 +106,8 @@ register_mlr3 = function() { # nocov start
   mlr_resamplings$add("repeated_spcv_tiles", ResamplingRepeatedSpCVTiles)
   mlr_resamplings$add("sptcv_cstf", ResamplingSptCVCstf)
   mlr_resamplings$add("repeated_sptcv_cstf", ResamplingRepeatedSptCVCstf)
+  mlr_resamplings$add("spcv_knndm", ResamplingSpCVKnndm)
+  mlr_resamplings$add("repeated_spcv_knndm", ResamplingRepeatedSpCVKnndm)
 }
 
 .onLoad = function(libname, pkgname) { # nolint
